//
// Revalation - a simple yet flexible relational database
// Copyright (C) 2000 Mark Rigby-Jones <mrj@compsoc.net>
// http://www.compsoc.net/~mrj/palm/revalation/
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <Pilot.h>
#include "revalation.h"
#include "revalationRsc.h"

/****************************************************************************
 *									    *
 *  New Database Dialog Box						    *
 *									    *
 ****************************************************************************/

void FormNewDbPopup(void) {
    FormPtr		frm;
    FormPtr		oldfrm;
    UInt		button;

    if (count < MAXITEMS) {
	frm = FrmInitForm(FormNewDb);
	oldfrm = FrmGetActiveForm();
	
        FrmSetEventHandler(frm, FormNewDbHandleEvent);
        FrmSetActiveForm(frm);
        FrmSetFocus(frm, FrmGetObjectIndex(frm, FieldNewDbName));
        button = FrmDoDialog(frm);
        FrmSetActiveForm(oldfrm);
        FrmDeleteForm(frm);
    
        if (button == ButtonNewDbOK) {
	    FrmGotoForm(FormTblList);
	}
    } else {
	FrmCustomAlert(AlertError,
		"You have already created the maximum number of databases.",
		NULL, NULL);
    }
}

Boolean FormNewDbHandleEvent(EventPtr e) {
    Boolean		handled = false;
    FormPtr		frm;
    FieldPtr		fld;
    CharPtr		txt;
    Err			error;
    VoidHand		h;
    LocalID		appinfo;
    AppInfoPtr		appinfoptr;
    UInt		newcardNo;
    LocalID		newdbID;
    DmOpenRef		newdatabase;

    InsPtEnable(true);

    switch (e->eType) {
	
      	case menuEvent:
    	    MenuEraseStatus(NULL);
    	    frm = FrmGetActiveForm();
    	    fld = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, FieldNewDbName));
    
    	    switch(e->data.menu.itemID) {
		case MenuItemEditUndo:
	    	    FldUndo(fld);
		    break;
		case MenuItemEditCopy:
		    FldCopy(fld);
		    break;
		case MenuItemEditCut:
		    FldCut(fld);
		    break;
		case MenuItemEditPaste:
		    FldPaste(fld);
		    break;
		case MenuItemEditSelect:
		    FldSetSelection(fld, 0, FldGetTextLength(fld));
		    break;
		case MenuItemEditKeyboard:
		    SysKeyboardDialog(kbdDefault);
		    break;
		case MenuItemEditGraffiti:
		    SysGraffitiReferenceDialog(referenceDefault);
		    break;
	    }

	    handled = true;
	    break;

	case ctlSelectEvent:
	    switch(e->data.ctlSelect.controlID) {
		case ButtonNewDbOK:
		    frm = FrmGetActiveForm();
		    fld = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm,
				FieldNewDbName));
		    txt = FldGetTextPtr(fld);

		    if (txt == NULL) {
			error = dmErrInvalidDatabaseName;
		    } else {
			newcardNo = 0;
			error = DmCreateDatabase (newcardNo, txt, CreatorID,
				DataDbID, false);
			if (!error) {
			    newdbID = DmFindDatabase(newcardNo, txt);
			    newdatabase = DmOpenDatabase(newcardNo, newdbID,
				    dmModeReadWrite);
			    h = DmNewHandle(newdatabase, sizeof(AppInfoType));
			    if (!h) {
				DmCloseDatabase(newdatabase);
				DmDeleteDatabase(newcardNo, newdbID);
				error = dmErrMemError;
			    } else {
				if (database) {
				    DmCloseDatabase(database);
				}
				cardNo = newcardNo;
				dbID = newdbID;
				database = newdatabase;
				appinfo = MemHandleToLocalID(h);
				DmSetDatabaseInfo(cardNo, dbID, NULL, NULL,
					NULL, NULL, NULL, NULL, NULL, &appinfo,
					NULL, NULL, NULL);
				appinfoptr = (AppInfoPtr)MemHandleLock(h);
				DmSet(appinfoptr, 0, sizeof(AppInfoType), 0);
				CategoryInitialize(appinfoptr,
					CategoriesDefault);
				MemPtrUnlock(appinfoptr);
			    }
			}
		    }
		    if (error) {
			if (error == dmErrAlreadyExists) {
			    FrmCustomAlert(AlertError,
				    "A databse with that name already exists.",
				    NULL, NULL);
			} else if (error == dmErrInvalidDatabaseName) {
			    FrmCustomAlert(AlertError,
				    "That is not a valid database name.",
				    NULL, NULL);
			} else {
			    FrmCustomAlert(AlertError,
				    "Unable to create a new database.",
				    NULL, NULL);
			}
			handled = true;
		    }
		    break;
	    }
	    break;

	default:
	    break;
    }

    return handled;
}
