//
// Revalation - a simple yet flexible relational database
// Copyright (C) 2000 Mark Rigby-Jones <mrj@compsoc.net>
// http://www.compsoc.net/~mrj/palm/revalation/
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <Pilot.h>
#include "revalation.h"
#include "revalationRsc.h"

/****************************************************************************
 *									    *
 *  New Table Dialog Box						    *
 *									    *
 ****************************************************************************/

void FormNewTablePopup(void) {
    FormPtr		frm;
    FormPtr		oldfrm;
    UInt		button;

    if (count < (dmRecNumCategories - 1)) {
	frm = FrmInitForm(FormNewTable);
	oldfrm = FrmGetActiveForm();
	
        FrmSetEventHandler(frm, FormNewTableHandleEvent);
        FrmSetActiveForm(frm);
        FrmSetFocus(frm, FrmGetObjectIndex(frm, FieldNewTableName));
        button = FrmDoDialog(frm);
        FrmSetActiveForm(oldfrm);
        FrmDeleteForm(frm);
    
        if (button == ButtonNewTableOK) {
	    FrmGotoForm(FormRecList);
	}
    } else {
	FrmCustomAlert(AlertError,
		"You have already created the maximum number of tables.",
		NULL, NULL);
    }
}

Boolean FormNewTableHandleEvent(EventPtr e) {
    Boolean		handled = false;
    Char		buf[dmCategoryLength];
    FormPtr		frm;
    FieldPtr		fld;
    CharPtr		txt;
    Err			error;
    Word		i;
    
    InsPtEnable(true);
    
    switch (e->eType) {
	case menuEvent:
	    MenuEraseStatus(NULL);
	    frm = FrmGetActiveForm();
	    fld = FrmGetObjectPtr(frm,
		    FrmGetObjectIndex(frm, FieldNewTableName));

	    switch(e->data.menu.itemID) {
		case MenuItemEditUndo:
		    FldUndo(fld);
		    break;
		case MenuItemEditCopy:
		    FldCopy(fld);
		    break;
		case MenuItemEditCut:
		    FldCut(fld);
		    break;
		case MenuItemEditPaste:
		    FldPaste(fld);
		    break;
		case MenuItemEditSelect:
		    FldSetSelection(fld, 0, FldGetTextLength(fld));
		    break;
		case MenuItemEditKeyboard:
		    SysKeyboardDialog(kbdDefault);
		    break;
		case MenuItemEditGraffiti:
		    SysGraffitiReferenceDialog(referenceDefault);
		    break;
	    }

	    handled = true;
	    break;

	case ctlSelectEvent:
	    switch(e->data.ctlSelect.controlID) {
		case ButtonNewTableOK: 
		    frm = FrmGetActiveForm();
		    fld = FrmGetObjectPtr(frm,
			    FrmGetObjectIndex(frm, FieldNewTableName));
		    txt = FldGetTextPtr(fld);
		    error = 0;
		    table = 0;
		    if ((txt == NULL) || (txt[0] == 0)) {
			error = dmErrInvalidDatabaseName;
		    } else {
			for (i = 0; i < dmRecNumCategories; i++) {
			    CategoryGetName(database, i, buf);
			    if ((table == 0) && (buf[0] == 0)) {
				table = i;
			    } else if (StrCompare(buf, txt) == 0) {
				error = dmErrAlreadyExists;
			    }
			}
		    }
		    if (error) {
			if (error == dmErrAlreadyExists) {
			    FrmCustomAlert(AlertError,
				    "A table with that name already exists.",
				    NULL, NULL);
			} else if (error == dmErrInvalidDatabaseName) {
			    FrmCustomAlert(AlertError,
				    "That is not a valid table name.",
				    NULL, NULL);
			}
			handled = true;
		    } else {
			CategorySetName(database, table, txt);
		    }
		    break;
	    }
	    break;

	default:
	    break;
    }

    return handled;
}
