//
// Revalation - a simple yet flexible relational database
// Copyright (C) 2000 Mark Rigby-Jones <mrj@compsoc.net>
// http://www.compsoc.net/~mrj/palm/revalation/
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <Pilot.h>
#include "revalation.h"
#include "revalationRsc.h"

/****************************************************************************
 *									    *
 *  Record List Form							    *
 *									    *
 ****************************************************************************/

Boolean FormRecListHandleEvent(EventPtr e) {
    Boolean		handled = false;
    FormPtr		frm;
    TablePtr		tbl;
    CharPtr		ptr;
    Short		i;
    Short		oldvalue;
    Err			error;
    
    switch (e->eType) {
	case frmOpenEvent:
	    frm = FrmGetActiveForm();
	    tbl = FrmGetObjectPtr(frm,
		    FrmGetObjectIndex(frm, TableRecListRecords));

	    ptr = FrmGetLabel(frm, LabelRecListName);
	    CategoryGetName (database, table, ptr);
	    ptr += StrLen(ptr);
	    ptr[0] = '@';
	    ptr++;
	    error = DmDatabaseInfo(cardNo, dbID, ptr, NULL, NULL, NULL, NULL,
		    NULL, NULL, NULL, NULL, NULL, NULL);

	    TblSetColumnUsable(tbl, 0, true);
	    TblSetCustomDrawProcedure(tbl, 0, TableRecListRecordsDrawItem);
	    for (i = 0; i < 11; i++) {
		TblSetItemStyle(tbl, i, 0, customTableItem);
	    }
	    TblHasScrollBar(tbl, true);
	    value = 0;

	    GetRecordList(frm);
	
   	    ScrollBarRecListRecordsSet(frm); 
	    FrmDrawForm(frm);
	    TblDrawTable(tbl);
	
	    handled = true;
	    break;
	
	case menuEvent:
	    MenuEraseStatus(NULL);

	    switch(e->data.menu.itemID) {
		case MenuItemRecListTableNew:
		    FormNewTablePopup();
		    break;
		case MenuItemRecListTableFields:
		    FormFldListPopup();
		    break;
		case MenuItemRecListTableRename:
		    FormRenameTablePopup();
		    break;
		case MenuItemRecListTableDelAll:
		    if (!prefs.deleteAll ||
			    (FrmCustomAlert(AlertConfirmDelete,
					    "all records from this table",
					    NULL, NULL) == 0)) {
			FrmCustomAlert(AlertError, "GNDN", NULL, NULL);
			// TODO the deletion
			// FrmGotoForm(FormRecList);
		    }
		    break;
		case MenuItemRecListTableDelete:
		    if (!prefs.deleteTable ||
			    (FrmCustomAlert(AlertConfirmDelete,
					    "this table",
					    NULL, NULL) == 0)) {
			FrmCustomAlert(AlertError, "GNDN", NULL, NULL);
			// TODO the deletion
			// FrmGotoForm(FormTblList);
		    }
		    break;
		case MenuItemRecListRecordNew:
		    record = -1;
		    FrmGotoForm(FormRecord);
		    break;
		case MenuItemRecListOptionsTablePrefs:
		    FormTablePrefsPopup();
		    break;
		case MenuItemRecListOptionsDbPrefs:
		    FormDbPrefsPopup();
		    break;
		case MenuItemRecListOptionsPreferences:
		    FormPreferencesPopup();
		    break;
		case MenuItemRecListOptionsAbout:
		    FrmAlert(AlertAbout);
		    break;
	    }

	    handled = true;
	    break;
	
	case sclRepeatEvent:
	    frm = FrmGetActiveForm();
	    tbl = FrmGetObjectPtr(frm,
		    FrmGetObjectIndex(frm, TableRecListRecords));
	    
	    value = e->data.sclRepeat.newValue;
	    TblDrawTable(tbl);
	    break;
	
	case ctlSelectEvent:
	    switch(e->data.ctlSelect.controlID) {
		case ButtonRecListDone:
		    FrmGotoForm(FormTblList);
		    break;
		case ButtonRecListNew:
		    record = -1;
		    FrmGotoForm(FormRecord);
		    break;
	    }
	    break;

	case tblSelectEvent:
	    switch(e->data.tblSelect.tableID) {
		case TableRecListRecords:
		    value += e->data.tblSelect.row;
		    record = items[value];
		    FrmGotoForm(FormRecord);
		    break;
	    }
	    break;

	case keyDownEvent:
	    oldvalue = value;

    	    switch(e->data.keyDown.chr) {
		case pageUpChr:
		    if (value < 11) {
			value = 0;
		    } else {
			value -= 10;
		    }
		    break;
		case pageDownChr:
		    if (value > (count - 22)) {
			value = count - 11;
		    } else {
			value += 10;
		    }
		    break;
	    }

	    if (value != oldvalue) {
		frm = FrmGetActiveForm();
		tbl = FrmGetObjectPtr(frm,
			FrmGetObjectIndex(frm, TableRecListRecords));

    		ScrollBarRecListRecordsSet(frm);
    		TblDrawTable(tbl);
    	    }
	
    	    break;

	default:
	    break;
    }

    return handled;
}

void GetRecordList(FormPtr frm) {
    CharPtr		label;

    count = DmNumRecordsInCategory (database, table);

    label = FrmGetLabel(frm, LabelRecListCount);
    if (count == 1) {
        StrCopy(label, "1 Record");
    } else if (count > 0) {
        StrIToA(label, count);
        StrCat(label, " Records");
    }
}

void TableRecListRecordsDrawItem(VoidPtr tbl, Word row, Word column,
	RectanglePtr bounds) {
    Char		buf[dmCategoryLength];
    RectangleType	clip;
    Err			error;
    UInt		index = 0;

    WinEraseRectangle(bounds, 0);
    if (row < count) {
	error = DmSeekRecordInCategory(database, &index, row + value,
		dmSeekForward, table);
	StrIToA(buf, index); // FIXME!
    	WinGetClip(&clip);
    	WinSetClip(bounds);
    	WinDrawChars(buf, StrLen(buf), bounds->topLeft.x, bounds->topLeft.y);
    	WinSetClip(&clip);
    } else {
	TblSetRowUsable(tbl, row, false);
    }
}

void ScrollBarRecListRecordsSet(FormPtr frm) {
    ScrollBarPtr	bar = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm,
				ScrollBarRecListRecords));
    Short		max = 0;

    if (count > 11) {
	max = count - 11;
    }
    SclSetScrollBar(bar, value, 0, max, 11);
}
