//
// Revalation - a simple yet flexible relational database
// Copyright (C) 2000 Mark Rigby-Jones <mrj@compsoc.net>
// http://www.compsoc.net/~mrj/palm/revalation/
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <Pilot.h>
#include "revalation.h"
#include "revalationRsc.h"

/****************************************************************************
 *									    *
 *  Rename Database Dialog Box						    *
 *									    *
 ****************************************************************************/

void FormRenameDbPopup(void) {
    Char		buf[dmDBNameLength];
    FormPtr		frm;
    FormPtr		oldfrm;
    FieldPtr		fld;
    Err			error;
    Word 		label;
    UInt		button;

    frm = FrmInitForm(FormRenameDb);
    oldfrm = FrmGetActiveForm();

    error = DmDatabaseInfo(cardNo, dbID, buf, NULL, NULL, NULL, NULL, NULL,
	    NULL, NULL, NULL, NULL, NULL);

    fld = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, FieldRenameDbName));
    FldInsert(fld, buf, StrLen(buf));
	
    FrmSetEventHandler(frm, FormRenameDbHandleEvent);
    FrmSetActiveForm(frm);
    FrmSetFocus(frm, FrmGetObjectIndex(frm, FieldRenameDbName));
    button = FrmDoDialog(frm);
    FrmSetActiveForm(oldfrm);
   
    if (button == ButtonRenameDbOK) {
    	label = FrmGetObjectIndex(oldfrm, LabelTblListName);
	FrmHideObject(oldfrm, label);
	error = DmDatabaseInfo(cardNo, dbID,
		FrmGetLabel(oldfrm, LabelTblListName),
		NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
	FrmShowObject(oldfrm, label);
    }

    FrmDeleteForm(frm);
}

Boolean FormRenameDbHandleEvent(EventPtr e) {
    Boolean		handled = false;
    FormPtr		frm;
    FieldPtr		fld;
    CharPtr		txt;
    Err			error;
    
    InsPtEnable(true);

    switch (e->eType) {
	case menuEvent:
	    MenuEraseStatus(NULL);
	    frm = FrmGetActiveForm();
	    fld = FrmGetObjectPtr(frm,
		    FrmGetObjectIndex(frm, FieldRenameDbName));

	    switch(e->data.menu.itemID) {
		case MenuItemEditUndo:
		    FldUndo(fld);
		    break;
		case MenuItemEditCopy:
		    FldCopy(fld);
		    break;
		case MenuItemEditCut:
		    FldCut(fld);
		    break;
		case MenuItemEditPaste:
		    FldPaste(fld);
		    break;
		case MenuItemEditSelect:
		    FldSetSelection(fld, 0, FldGetTextLength(fld));
		    break;
		case MenuItemEditKeyboard:
		    SysKeyboardDialog(kbdDefault);
		    break;
		case MenuItemEditGraffiti:
		    SysGraffitiReferenceDialog(referenceDefault);
		    break;
	    }

	    handled = true;
	    break;

	case ctlSelectEvent:
	    switch(e->data.ctlSelect.controlID) {
		case ButtonRenameDbOK:
		    frm = FrmGetActiveForm();
		    fld = FrmGetObjectPtr(frm,
			    FrmGetObjectIndex(frm, FieldRenameDbName));
		    txt = FldGetTextPtr(fld);

		    if (txt == NULL) {
			error = dmErrInvalidDatabaseName;
		    } else {
			error = DmSetDatabaseInfo (cardNo, dbID, txt,
				NULL, NULL, NULL, NULL, NULL, NULL, NULL,
				NULL, NULL, NULL);
		    }
		    if (error) {
			if (error == dmErrAlreadyExists) {
			    FrmCustomAlert(AlertError,
				    "A databse with that name already exists.",
				    NULL, NULL);
			} else {
			    FrmCustomAlert(AlertError,
				    "That is not a valid database name.",
				    NULL, NULL);
			}
			handled = true;
		    }
		    break;
	    }
	    break;

	default:
	    break;
    }

    return handled;
}
