//
// Revalation - a simple yet flexible relational database
// Copyright (C) 2000 Mark Rigby-Jones <mrj@compsoc.net>
// http://www.compsoc.net/~mrj/palm/revalation/
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <Pilot.h>
#include "revalation.h"
#include "revalationRsc.h"

/****************************************************************************
 *									    *
 *  Declaration of the global varaibles					    *
 *									    *
 ****************************************************************************/

RevalationPrefsType	prefs;			// Application preferences
UInt			count = 0;		// No. items in table
UInt			value = 0;		// Top visible table item
unsigned long		items[MAXITEMS];	// Table items
unsigned long		moreitems[MAXITEMS];	// Additional data for above
UInt			cardNo;			// Card no. of active db
LocalID			dbID;			// Local ID of active db
DmOpenRef		database = NULL;	// Active database
Word			table;			// Selected table
UInt			record;			// Selected record

/****************************************************************************
 *									    *
 *  Main Program Body							    *
 *									    *
 ****************************************************************************/

DWord PilotMain(Word cmd, Ptr cmdPBP, Word launchFlags) {
    Word		err;
    DWord		romversion;

    if (cmd == sysAppLaunchCmdNormalLaunch) {
        FtrGet(sysFtrCreator, sysFtrNumROMVersion, &romversion);
        if (romversion < ROMVersion20) {
            return 1;
        }

	err = StartApplication();
	if (err) {
	    return err;
	}

	EventLoop();
	StopApplication();
    } else {
	return sysErrParamErr;
    }

    return 0;
}

Word StartApplication(void) {
    Word		prefsSize = sizeof(RevalationPrefsType);

    if (PrefGetAppPreferences(CreatorID, 0, &prefs, &prefsSize, true)
	    == noPreferenceFound) {
	prefs.deleteDatabase = true;
	prefs.deleteData = true;
	prefs.deleteTable = true;
	prefs.deleteAll = true;
	prefs.deleteRecord = false;
	prefs.rememberLastView = true;
    }

    FrmGotoForm(FormDbList);
    return 0;
}

void EventLoop(void) {
    Word		err;
    EventType		e;

    do {
	EvtGetEvent(&e, evtWaitForever);
	if (!SysHandleEvent(&e)) {
	    if (!MenuHandleEvent(NULL, &e, &err)) {
		if (!ApplicationHandleEvent(&e)) {
		    FrmDispatchEvent(&e);
		}
	    }
	}
    } while (e.eType != appStopEvent);
}

void StopApplication(void) {
    PrefSetAppPreferences(CreatorID, 0, 0, &prefs, sizeof(RevalationPrefsType),
	    true);

    FrmSaveAllForms();
    FrmCloseAllForms();

    if (database) {
	DmCloseDatabase(database);
	database = NULL;
    }
}

/****************************************************************************
 *									    *
 *  Main Event Handler							    *
 *									    *
 ****************************************************************************/

Boolean ApplicationHandleEvent(EventPtr e) {
    FormPtr		frm;
    Word		formId;
    Boolean		handled = false;

    if (e->eType == frmLoadEvent) {
	formId = e->data.frmLoad.formID;
	frm = FrmInitForm(formId);
	FrmSetActiveForm(frm);

	switch(formId) {
	    case FormDbList:
	        FrmSetEventHandler(frm, FormDbListHandleEvent);
	    	break;
	    case FormTblList:
	        FrmSetEventHandler(frm, FormTblListHandleEvent);
	    	break;
	    case FormRecList:
	        FrmSetEventHandler(frm, FormRecListHandleEvent);
	    	break;
	    case FormRecord:
	        FrmSetEventHandler(frm, FormRecordHandleEvent);
	    	break;
	}
	handled = true;
    }

    return handled;
}
