//
// Revalation - a simple yet flexible relational database
// Copyright (C) 2000 Mark Rigby-Jones <mrj@compsoc.net>
// http://www.compsoc.net/~mrj/palm/revalation/
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

//
// Type Definitions
//

typedef struct {
    Boolean			deleteDatabase;
    Boolean			deleteData;
    Boolean			deleteTable;
    Boolean			deleteAll;
    Boolean			deleteRecord;
    Boolean			rememberLastView;
} RevalationPrefsType;

enum revalationFieldTypes {
    booleanFieldType,
    integerFieldType,
    textFieldType/*,
    directFieldType,
    indirectFieldType*/
};

typedef enum revalationFieldTypes RevalationFieldTypeType;

typedef struct {
    Word			order;
    RevalationFieldTypeType	type;
    Char			name[32];
    Word			directTable;
    Word			indirectTable;
} RevalationFieldType;

//
// Constants
// 

#define CreatorID 'mrjR'
#define DataDbID 'Data'

#define MAXITEMS 32

#define ROMVersion20 0x02003000

//
// Global Variables
//

extern RevalationPrefsType	prefs;
extern UInt			count;
extern UInt			value;
extern unsigned long		items[MAXITEMS];
extern unsigned long		moreitems[MAXITEMS];
extern UInt			cardNo;
extern LocalID			dbID;
extern DmOpenRef		database;
extern Word			table;
extern UInt			record;


//
// Function Definitions
//

// Main Program Body

extern DWord PilotMain(Word cmd, Ptr cmdPBP, Word launchFlags);
extern Word StartApplication(void);
extern void EventLoop(void);
extern void StopApplication(void);

// Main Event Handler

extern Boolean ApplicationHandleEvent(EventPtr e);

// Database List From

extern Boolean FormDbListHandleEvent(EventPtr e);
extern void GetDatabaseList(FormPtr frm);
extern void TableDbListDatabasesDrawItem(VoidPtr tbl, Word row, Word column,
	RectanglePtr bounds);
extern void ScrollBarDbListDatabasesSet(FormPtr frm);

// Preferences Dialog Box

extern void FormPreferencesPopup(void);

// New Database Dialog Box

extern void FormNewDbPopup(void);
extern Boolean FormNewDbHandleEvent(EventPtr e);

// Table List Form

extern Boolean FormTblListHandleEvent(EventPtr e);
extern void GetTableList(FormPtr frm);
extern void TableTblListTablesDrawItem(VoidPtr tbl, Word row, Word column,
	RectanglePtr bounds);
extern void ScrollBarTblListTablesSet(FormPtr frm);

// Rename Database Dialog Box

extern void FormRenameDbPopup(void);
extern Boolean FormRenameDbHandleEvent(EventPtr e);

// Database Preferences Dialog Box

extern void FormDbPrefsPopup(void);

// New Table Dialog Box

extern void FormNewTablePopup(void);
extern Boolean FormNewTableHandleEvent(EventPtr e);

// Record List Form

extern Boolean FormRecListHandleEvent(EventPtr e);
extern void GetRecordList(FormPtr frm);
extern void TableRecListRecordsDrawItem(VoidPtr tbl, Word row, Word column,
	RectanglePtr bounds);
extern void ScrollBarRecListRecordsSet(FormPtr frm);

// Rename Table Dialog Box

extern void FormRenameTablePopup(void);
extern Boolean FormRenameTableHandleEvent(EventPtr e);

// Table Preferences Dialog Box

extern void FormTablePrefsPopup(void);

// Field List Dialog Box

extern void FormFldListPopup(void);
extern Boolean FormFldListHandleEvent(EventPtr e);
extern void GetFieldList(void);
extern void FreeFieldList(void);
extern void ScrollBarFldListFieldsSet(FormPtr frm);

// Record View / Edit Dialog Box

extern Boolean FormRecordHandleEvent(EventPtr e);
