//
// Revalation - a simple yet flexible relational database
// Copyright (C) 2000 Mark Rigby-Jones <mrj@compsoc.net>
// http://www.compsoc.net/~mrj/palm/revalation/
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <Pilot.h>
#include "revalation.h"
#include "revalationRsc.h"

/****************************************************************************
 *									    *
 *  Table List Form							    *
 *									    *
 ****************************************************************************/

Boolean FormTblListHandleEvent(EventPtr e) {
    Boolean		handled = false;
    FormPtr		frm;
    TablePtr		tbl;
    Short		i;
    Short		oldvalue;
    Err			error;
    
    switch (e->eType) {
	case frmOpenEvent:
	    frm = FrmGetActiveForm();
	    tbl = FrmGetObjectPtr(frm,
		    FrmGetObjectIndex(frm, TableTblListTables));

	    error = DmDatabaseInfo(cardNo, dbID,
		    FrmGetLabel(frm, LabelTblListName), NULL, NULL, NULL, NULL,
		    NULL, NULL, NULL, NULL, NULL, NULL);

	    TblSetColumnUsable(tbl, 0, true);
	    TblSetCustomDrawProcedure(tbl, 0, TableTblListTablesDrawItem);
	    for (i = 0; i < 11; i++) {
		TblSetItemStyle(tbl, i, 0, customTableItem);
	    }
	    TblHasScrollBar(tbl, true);
	    value = 0;

	    GetTableList(frm);
	
   	    ScrollBarTblListTablesSet(frm); 
	    FrmDrawForm(frm);
	    TblDrawTable(tbl);
	
	    handled = true;
	    break;
	
	case menuEvent:
	    MenuEraseStatus(NULL);

	    switch(e->data.menu.itemID) {
		case MenuItemTblListDatabaseNew:
		    FormNewDbPopup();
		    break;
		case MenuItemTblListDatabaseRename:
		    FormRenameDbPopup();
		    break;
		case MenuItemTblListDatabaseDelAll:
		    if (!prefs.deleteData ||
			    (FrmCustomAlert(AlertConfirmDelete,
					    "all data from this database",
					    NULL, NULL) == 0)) {
			FrmCustomAlert(AlertError, "GNDN", NULL, NULL);
			// TODO the deletion
			// FrmGotoForm(FormTblList);
		    }
		    break;
		case MenuItemTblListDatabaseDelete:
		    if (!prefs.deleteDatabase ||
			    (FrmCustomAlert(AlertConfirmDelete,
					    "this database",
					    NULL, NULL) == 0)) {
			error = DmCloseDatabase(database);
			database = NULL;
			error = DmDeleteDatabase(cardNo, dbID);
			if (error) {
			    FrmCustomAlert(AlertError,
				    "Unable to delete the database!",
				    NULL, NULL);
			}
			FrmGotoForm(FormDbList);
		    }
		    break;
		case MenuItemTblListTableNew:
		    FormNewTablePopup();
		    break;
		case MenuItemTblListOptionsDbPrefs:
		    FormDbPrefsPopup();
		    break;
		case MenuItemTblListOptionsPreferences:
		    FormPreferencesPopup();
		    break;
		case MenuItemTblListOptionsAbout:
		    FrmAlert(AlertAbout);
		    break;
	    }

	    handled = true;
	    break;
	
	case sclRepeatEvent:
	    frm = FrmGetActiveForm();
	    tbl = FrmGetObjectPtr(frm,
		    FrmGetObjectIndex(frm, TableTblListTables));
	    
	    value = e->data.sclRepeat.newValue;
	    TblDrawTable(tbl);
	    break;
	
	case ctlSelectEvent:
	    switch(e->data.ctlSelect.controlID) {
		case ButtonTblListDone:
		    error = DmCloseDatabase(database);
		    database = NULL;
		    FrmGotoForm(FormDbList);
		    break;
		case ButtonTblListNew:
		    FormNewTablePopup();
		    break;
	    }
	    break;

	case tblSelectEvent:
	    switch(e->data.tblSelect.tableID) {
		case TableTblListTables:
		    value += e->data.tblSelect.row;
		    table = items[value];
		    FrmGotoForm(FormRecList);
		    break;
	    }
	    break;

	case keyDownEvent:
	    oldvalue = value;

    	    switch(e->data.keyDown.chr) {
		case pageUpChr:
		    if (value < 11) {
			value = 0;
		    } else {
			value -= 10;
		    }
		    break;
		case pageDownChr:
		    if (value > (count - 22)) {
			value = count - 11;
		    } else {
			value += 10;
		    }
		    break;
	    }

	    if (value != oldvalue) {
		frm = FrmGetActiveForm();
		tbl = FrmGetObjectPtr(frm,
			FrmGetObjectIndex(frm, TableTblListTables));

    		ScrollBarTblListTablesSet(frm);
    		TblDrawTable(tbl);
    	    }
	
    	    break;

	default:
	    break;
    }

    return handled;
}

void GetTableList(FormPtr frm) {
    CharPtr		label;
    Char		buf[dmCategoryLength];
    Word		i;

    count = 0;
    for (i = 0; i < dmRecNumCategories; i++) {
	CategoryGetName(database, i, buf);
	if ((i != dmUnfiledCategory) && (buf[0] > 0)) {
	    items[count] = i;
	    count++;
	}
    }

    label = FrmGetLabel(frm, LabelTblListCount);
    if (count == 1) {
        StrCopy(label, "1 Table");
    } else if (count > 0) {
        StrIToA(label, count);
        StrCat(label, " Tables");
    }
}

void TableTblListTablesDrawItem(VoidPtr tbl, Word row, Word column,
	RectanglePtr bounds) {
    Char		buf[dmCategoryLength];
    RectangleType	clip;

    WinEraseRectangle(bounds, 0);
    if (row < count) {
	CategoryGetName(database, items[row + value], buf);
    	WinGetClip(&clip);
    	WinSetClip(bounds);
    	WinDrawChars(buf, StrLen(buf), bounds->topLeft.x, bounds->topLeft.y);
    	WinSetClip(&clip);
    } else {
	TblSetRowUsable(tbl, row, false);
    }
}

void ScrollBarTblListTablesSet(FormPtr frm) {
    ScrollBarPtr	bar = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm,
				ScrollBarTblListTables));
    Short		max = 0;
    
    if (count > 11) {
	max = count - 11;
    }
    SclSetScrollBar(bar, value, 0, max, 11);
}
